# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# Copyright (C) 2015 - Murilo Faria
# Original Version: 1.0.5
#
# Modified Version: 1.0.6 - Davide Rassu, sept. 2016

import ac, acsys
import platform, os, sys
import configparser, traceback

if platform.architecture()[0] == "64bit":
    sysdir = "stdlib64"
else:
    sysdir = "stdlib"

sys.path.insert(0, os.path.join(os.path.dirname(__file__), sysdir))
os.environ['PATH'] = os.environ['PATH'] + ";."

from sim_info import info

app = 0
app_size_x = 340
app_size_y = 145
background = 0
labels = {}
cars = 30
inidir = "apps/python/splits/"
bestSplitsFileName  = "splits.ini"
config = None
track = None
car = None
dic = {}
sectors = [0,0,0]
hasSection = False
currentSector = None
lastLap = 0

colors = {
	"white"      : [255, 255, 255],
	"yellow"     : [247, 183,   9],
	"green"      : [2,   221,  20],
	"red"        : [252,  35,  35],
	"orange"     : [255, 102,   0],
	"azure"      : [51,  102, 153],
    "purple"     : [238, 130, 238]
}

def acMain(ac_version):
    global app, track, car

    try:
        app = ac.newApp("Splits")
        ac.setTitle(app, "")		
        ac.drawBorder(app, 0)
        ac.setIconPosition(app, 0, -10000)		
        ac.setSize(app, app_size_x, app_size_y)
        ac.setBackgroundOpacity(app, background)		
	
        renderLabels()

        if track is None:
            track = ac.getTrackName(0);

        if car is None:
            car = ac.getCarName(0)
        
        calculateOptimalLap()
        loadBestLap()
        	
        return "Splits"
		
    except:
        exc_type, exc_value, exc_traceback = sys.exc_info()
        ac.console('Splits Error (logged to file)')
        ac.log(repr(traceback.format_exception(exc_type, exc_value, exc_traceback)))

def createLabel(name, text, x, y, labelColor):
    global app, labels

    label = ac.addLabel(app, name)
    ac.setText(label, text)
    ac.setPosition(label, x, y)
    ac.setFontColor(label, *rgb(colors[labelColor]))	
	
    labels[name] = label		
		
def renderLabels():
    global labels
		
    createLabel("emptyLabel", "", 5, 30, "green")
    createLabel("sector1TitleLabel", "-- 1 --", 80, 30, "green")
    createLabel("sector2TitleLabel", "-- 2 --", 140, 30, "green")
    createLabel("sector3TitleLabel", "-- 3 --", 200, 30, "green")
    createLabel("totalTitleLabel", "--  Lap  --", 260, 30, "green")

    createLabel("currentLapTitleLabel", "Current", 5, 50, "red")		
    createLabel("currentSector1Label", "00.000", 80, 50, "white")
    createLabel("currentSector2Label", "00.000", 140, 50, "white")
    createLabel("currentSector3Label", "00.000", 200, 50, "white")
    createLabel("currentLapLabel", "00:00:000", 260, 50, "white")
	
    createLabel("lastLapTitleLabel", "Last", 5, 70, "red")		
    createLabel("lastSector1Label", "00.000", 80, 70, "white")
    createLabel("lastSector2Label", "00.000", 140, 70, "white")
    createLabel("lastSector3Label", "00.000", 200, 70, "white")
    createLabel("lastLapLabel", "00:00:000", 260, 70, "white")
		
    createLabel("optiomalLapTitleLabel", "Optimal", 5, 90, "red")		
    createLabel("optimalSector1Label", "00.000", 80, 90, "orange")
    createLabel("optimalSector2Label", "00.000", 140, 90, "orange")
    createLabel("optimalSector3Label", "00.000", 200, 90, "orange")
    createLabel("optimalLapLabel", "00:00:000", 260, 90, "orange")		
	
    createLabel("mybestLapTitleLabel", "Best", 5, 120, "red")
    createLabel("mybestLapSector1Label", "00.000", 80, 120, "azure")
    createLabel("mybestLapSector2Label", "00.000", 140, 120, "azure")
    createLabel("mybestLapSector3Label", "00.000", 200, 120, "azure")
    createLabel("mybestLapLabel", "00:00:000", 260, 120, "azure")

def timeToMinSecMsecTuple(t):
    mins = t // (60*1000)
    secs = (t - 60*1000*mins) // 1000
    msecs = (t - 60*1000*mins - secs*1000)
    return (mins, secs, msecs)

def formatTime(t):
    mins, secs, msecs = timeToMinSecMsecTuple(abs(t))

    time = "%02d:%02d.%03d" % (mins, secs, msecs)

    return time

def formatSplit(t):
    mins, secs, msecs = timeToMinSecMsecTuple(abs(t))

    if mins > 0:
        secs = (mins * 60) + secs

    return "%02d.%03d" % (secs, msecs)


def getConfigFile():
    global config, inidir, bestSplitsFileName

    if config is None:
        config = configparser.RawConfigParser()
        config.read(inidir+bestSplitsFileName)
        
    return config

def getTrackAndCarSectionName():
    global hasSection, track, car
    config = getConfigFile()
    section = track+"#"+car

    if not hasSection:
        if not config.has_section(section):
            config.add_section(section)
            writeConfigFile(config)
        hasSection = True
        

    return section

def writeConfigFile(config):
    global inidir, bestSplitsFileName

    with open(inidir+bestSplitsFileName, 'w') as configfile:
        config.write(configfile)
    

def updateOptimalSplitData(split, splitTime):
    global dic, track, car

    config = getConfigFile()
    section = getTrackAndCarSectionName()
    splitAttr = "OS%d"%(split)
    optimalSplit = dic.get(splitAttr, 0)

    if optimalSplit == 0 and config.has_option(section, splitAttr):
        optimalSplit = config.getint(section, splitAttr)

    if splitTime > 0 and (optimalSplit == 0 or splitTime < optimalSplit):
        optimalSplit = splitTime
        config.set(section, splitAttr, optimalSplit)
        writeConfigFile(config)

    dic[splitAttr] = optimalSplit

    return optimalSplit


def calculateOptimalLap():
    global dic, track, car

    optimalLap = 0

    for i in range(1, 4):
        splitAttr = "OS%d"%(i)
        split = dic.get(splitAttr, 0)

        if split == 0:
            config = getConfigFile()
            section = getTrackAndCarSectionName()
            if config.has_option(section, splitAttr):
                split = config.getint(section, splitAttr)
                dic[splitAttr] = split
            
        
        optimalLap = optimalLap + split

        label = None

        if i == 1:
            label = labels["optimalSector1Label"]
        elif i == 2:
            label = labels["optimalSector2Label"]
        elif i == 3:
            label = labels["optimalSector3Label"]

        if label is not None:
            ac.setText(label, formatSplit(split))


    label = labels["optimalLapLabel"]
    ac.setText(label, formatTime(optimalLap))

def loadBestLap():
    global dic, track, car

    
    for i in range(1, 4):
        splitAttr = "MBLS%d"%(i)
        split = dic.get(splitAttr, 0)

        if split == 0:
            config = getConfigFile()
            section = getTrackAndCarSectionName()
            if config.has_option(section, splitAttr):
                split = config.getint(section, splitAttr)
                dic[splitAttr] = split
            
        label = None

        if i == 1:
            label = labels["mybestLapSector1Label"]
        elif i == 2:
            label = labels["mybestLapSector2Label"]
        elif i == 3:
            label = labels["mybestLapSector3Label"]

        if label is not None:
            ac.setText(label, formatSplit(split))


    splitAttr = "BL"
    bestLap = dic.get(splitAttr, 0)

    if bestLap == 0 and config.has_option(section, splitAttr):
        bestLap = config.getint(section, splitAttr)

    dic[splitAttr] = bestLap


    label = labels["mybestLapLabel"]
    ac.setText(label, formatTime(bestLap))


def getBestLapSplit(split):
    config = getConfigFile()
    section = getTrackAndCarSectionName()
    splitAttr = "MBLS%d"%(split)
    myBestSplit = dic.get(splitAttr, 0)

    if myBestSplit == 0 and config.has_option(section, splitAttr):
        myBestSplit = config.getint(section, splitAttr)

    return myBestSplit
    

def updateBestLapData(carId, lap, splits):
    global dic, track, car

    config = getConfigFile()
    section = getTrackAndCarSectionName()
    splitAttr = "BL"
    bestLap = dic.get(splitAttr, 0)

    if bestLap == 0 and config.has_option(section, splitAttr):
        bestLap = config.getint(section, splitAttr)
	
    isBestLap = False

    if lap > 0 and (bestLap == 0 or lap < bestLap):
        isBestLap = True
        bestLap = lap
        config.set(section, splitAttr, bestLap)
        writeConfigFile(config)
            
    dic[splitAttr] = bestLap

    label = labels["mybestLapLabel"]
    ac.setText(label, formatTime(bestLap))

    if isBestLap:
        s = 1
    
        for split in splits:
            updateBestLapSplit(carId, s, split)
            s = s + 1


def updateBestLapSplit(carId, split, splitTime):
    global dic, track, car

    config = getConfigFile()
    section = getTrackAndCarSectionName()
    splitAttr = "MBLS%d"%(split)
    myBestSplit = dic.get(splitAttr, 0)

    if myBestSplit == 0 and config.has_option(section, splitAttr):
        myBestSplit = config.getint(section, splitAttr)
    
    if splitTime > 0:
        myBestSplit = splitTime
        config.set(section, splitAttr, myBestSplit)
        writeConfigFile(config)

    dic[splitAttr] = myBestSplit

    label = None

    if split == 1:
        label = labels["mybestLapSector1Label"]
    elif split == 2:
        label = labels["mybestLapSector2Label"]
    elif split == 3:
        label = labels["mybestLapSector3Label"]

    if label is not None:
        ac.setText(label, formatSplit(myBestSplit))

   
    

def updateLastLapData(carId, lastLap, lastSplits):
    

    s = 1

       
    for split in lastSplits:

        if split == 0:
            hasAllSectors = False

        if s == 1:
            lastSector1Label = labels["lastSector1Label"]
            ac.setText(lastSector1Label, formatSplit(split))

            currentSector1Label = labels["currentSector1Label"]
            ac.setText(currentSector1Label, "00.000")
            ac.setFontColor(currentSector1Label, *rgb(colors["white"]))
            
        elif s == 2:
            lastSector2Label = labels["lastSector2Label"]
            ac.setText(lastSector2Label, formatSplit(split))

            currentSector2Label = labels["currentSector2Label"]
            ac.setText(currentSector2Label, "00.000")
            ac.setFontColor(currentSector2Label, *rgb(colors["white"]))
            
        elif s == 3:
            lastSector3Label = labels["lastSector3Label"]
            ac.setText(lastSector3Label, formatSplit(split))

            currentSector3Label = labels["currentSector3Label"]
            ac.setText(currentSector3Label, "00.000")
            ac.setFontColor(currentSector3Label, *rgb(colors["white"]))

        s = s + 1

    updateBestLapData(carId, lastLap, lastSplits)
    
    lastLapLabel = labels['lastLapLabel']
    ac.setText(lastLapLabel, formatTime(lastLap))

    currentLapLabel = labels["currentLapLabel"]
    ac.setText(currentLapLabel, "00:00:000")
    
    

def updateSplits(carId):
    global sectors, currentSector, lastLap

    currentTime = info.graphics.iCurrentTime
    normalizedSplinePosition = round(info.graphics.normalizedCarPosition, 4)

    isLapEnd = False

    if lastLap != info.graphics.iLastTime:
        lastLap = info.graphics.iLastTime
        isLapEnd = True
    
    hasAllSectors = False

    if currentSector is None:
        currentSector = 1
  

    currentSector1Label = labels["currentSector1Label"]
    currentSector2Label = labels["currentSector2Label"]
    currentSector3Label = labels["currentSector3Label"]
    

    if currentSector == 1 and normalizedSplinePosition == 0.3300:
        currentSector = 2
        sectors[0] = currentTime
        ac.setText(currentSector1Label, formatSplit(sectors[0]))
        optimalSplit = updateOptimalSplitData(1, sectors[0])
        bestLapSplit = getBestLapSplit(1)

        if optimalSplit == sectors[0]:
            ac.setFontColor(currentSector1Label, *rgb(colors["purple"]))
        elif sectors[0] < bestLapSplit:
            ac.setFontColor(currentSector1Label, *rgb(colors["azure"]))
        else:
            ac.setFontColor(currentSector1Label, *rgb(colors["white"]))
            
    elif currentSector == 2 and normalizedSplinePosition == 0.6600:
        currentSector = 3
        sector1 = sectors[0]
        sectors[1] = currentTime - sector1
        ac.setText(currentSector2Label, formatSplit(sectors[1]))
        optimalSplit = updateOptimalSplitData(2, sectors[1])
        bestLapSplit = getBestLapSplit(2)

        if optimalSplit == sectors[1]:
            ac.setFontColor(currentSector2Label, *rgb(colors["purple"]))
        elif sectors[1] < bestLapSplit:
            ac.setFontColor(currentSector2Label, *rgb(colors["azure"]))
        else:
            ac.setFontColor(currentSector2Label, *rgb(colors["white"]))
        
    elif currentSector == 3 and isLapEnd:
        sector1 = sectors[0]
        sector2 = sectors[1]
        sectors[2] = lastLap - (sector1 + sector2)
        sector3 = sectors[2]
        ac.setText(currentSector3Label, formatSplit(sectors[1]))
        currentLap = sector1 + sector2 + sector3
        hasAllSectors = True
        optimalSplit = updateOptimalSplitData(3, sectors[2])
        updateLastLapData(carId, lastLap, sectors)
        bestLapSplit = getBestLapSplit(2)
        currentSector = None
        isLapEnd = False
        sectors = [0,0,0]


    calculateOptimalLap()
        
  
	
def updateData(deltaT):
    global track, car, labels, normalizedSplinePosition

    try:

        if track is None:
            track = ac.getTrackName(0);

        if car is None:
            car = ac.getCarName(0)

        updateSplits(0)
              
                        
    except:
        exc_type, exc_value, exc_traceback = sys.exc_info()
        ac.console('Splits Error (logged to file)')
        ac.log(repr(traceback.format_exception(exc_type, exc_value, exc_traceback)))

def rgb(color, a = 1, bg = False):
	r = color[0] / 255
	g = color[1] / 255
	b = color[2] / 255
	if bg == False:
		return r, g, b, a
	else:
		return r, g, b


	
def acUpdate(deltaT):

    updateData(deltaT)
    
